
#pragma once

// disable "default constructor could not be generated" warning
#pragma warning(push)
#pragma warning(disable: 4510)
#pragma warning(disable: 4610)

#include <boost/mpl/and.hpp>
#include <boost/mpl/apply.hpp>
#include <boost/mpl/bool.hpp>
#include <boost/mpl/eval_if.hpp>
#include <boost/mpl/placeholders.hpp>
#include <typeinfo>
#include "is_debug.hpp"

namespace ketchup {

	namespace detail {

	// Note: if inline missed, the compiler doesn't remove s_count;
	inline int& get_trace_indent_count()
	{
		static int s_count = 0;
		return s_count;
	}

	template< class T >
	struct trace_base_on
	{
		explicit trace_base_on(LPCTSTR str)
		{
			for (int i = 0; i < get_trace_indent_count(); ++i) {
				ATLTRACE(_T(" "));
			}
			// Note: For the compiler, it seems better to define a temporary tid.
			const std::type_info& tid = typeid(T);
			ATLTRACE(tid.name());
			ATLTRACE(_T("::"));
			ATLTRACE(str);
			ATLTRACE(_T("\n"));
			++get_trace_indent_count();
		}

		~trace_base_on()
		{
			--get_trace_indent_count();
		}

		typedef trace_base_on type;
	};

	template< class T >
	struct trace_base_off
	{
		explicit trace_base_off(LPCTSTR) { }
		
		typedef trace_base_off type;
	};

	template< class T >
	struct get_trace_base
	{
		typedef typename boost::mpl::eval_if<
			boost::mpl::and_<ketchup::is_debug, typename T::trace_flag>,
			trace_base_on<boost::mpl::_>,
			trace_base_off<boost::mpl::_>
		>::type func;

		typedef typename boost::mpl::apply<func, T>::type type;
	};
	
	} // namespace detail

template< class T >
struct trace : detail::get_trace_base<T>::type
{
	typedef typename detail::get_trace_base<T>::type base_type;
	explicit trace(LPCTSTR str) : base_type(str) { }
};

#pragma warning(pop)

} // namespace ketchup
